@@ -7,7 +7,7 @@ use warnings FATAL => 'all';				# Enable warnings to catch errors
 
 # Initialize our version
 # $Revision: 1181 $
-our $VERSION = '1.21';
+our $VERSION = '1.22';
 
 # Import what we need from the POE namespace
 use POE;
@@ -827,9 +827,15 @@ sub Request_Output {
    if (defined $response->{'STREAM'}) {
       # Keep track if we plan to stream ...   	
    	if ( $_[HEAP]->{'RESPONSES'}->{ $id } ) {
+   	   if ( DEBUG ) {
+   	      warn "Restoring response from HEAP and id $id ";
+   	   }
    		$response = $_[HEAP]->{'RESPONSES'}->{ $id };
    	}
    	else {
+   	   if ( DEBUG ) {
+   	      warn "Saving HEAP response to id $id ";
+   	   }
          $_[HEAP]->{'RESPONSES'}->{ $id } = $response;
       }
    }
@@ -894,28 +900,52 @@ sub Request_Output {
       # Loops through current streams
       foreach (keys %{ $_[HEAP]->{'RESPONSES'} }) {
 
+         # Preliminary check
+      	if ( ! defined $_[HEAP]->{'REQUESTS'}->{ $_ }->[0] 
+      	      or ! defined $_[HEAP]->{'REQUESTS'}->{ $_ }->[0]->[ POE::Wheel::ReadWrite::HANDLE_INPUT ] ) {
+      		if ( DEBUG ) {
+      			warn 'Tried to send data over a closed/nonexistant socket!';
+      		}
+      		next;
+      	}
    	
-            # Sets the correct POE::Filter
-            unless (defined $response->{'IS_STREAMING'}) {
-            	# Mark this socket done
-            	$_[HEAP]->{'REQUESTS'}->{ $id }->[1] = 2;
-            	
-            	#
-               $_[HEAP]->{'REQUESTS'}->{ $_ }->[0]->set_output_filter(POE::Filter::Stream->new() ) ;
-               $response->is_streaming(1);
-            }
-            
-            if ( DEBUG ) {
-         		print "Sending stream via ".$response->{STREAM}." to $_ with id $id \n" ;
-         	}
-            
-            $kernel->yield(
+         # Sets the correct POE::Filter
+         unless (defined $response->{'IS_STREAMING'}) {
+         	# Mark this socket done
+         	$_[HEAP]->{'REQUESTS'}->{ $id }->[1] = 2;
+         	
+         	#
+            $_[HEAP]->{'REQUESTS'}->{ $_ }->[0]->set_output_filter(POE::Filter::Stream->new() ) ;
+            $response->is_streaming(1);
+         }
+         
+         if ( DEBUG ) {
+      		print "Sending stream via ".$response->{STREAM_SESSION}."/".$response->{STREAM}." to $_ with id $id \n" ;
+      	}
+         
+         my %stream = (
+            wheel    => $_[HEAP]->{'REQUESTS'}->{ $_ }->[0],  # wheel
+            request  => $_[HEAP]->{'REQUESTS'}->{ $id }->[3],  # request
+            response => $_[HEAP]->{'RESPONSES'}->{$_},         # response
+            id       => $_                                     # id
+         );
+         
+         # we send the event to stream with wheels request and response to the session 
+         # that has registered the streaming event (can be our own session ? it is left
+         # it because initially it was a simple $kernel->post)            
+   	   if (defined $response->{STREAM_SESSION}) {
+            POE::Kernel->post(
+               $response->{STREAM_SESSION},           # callback session
                $response->{STREAM},                   # callback event
-               $_[HEAP]->{'REQUESTS'}->{ $_ }->[0],   # wheel
-               $_[HEAP]->{'REQUESTS'}->{ $id }->[3],  # request
-               $_[HEAP]->{'RESPONSES'}->{$_},         # response
-               $_                                     # id
-            );
+               \%stream
+            );  
+         }
+         else {
+            POE::Kernel->post(
+               $response->{STREAM},                   # callback event
+               \%stream
+            );  
+         }
    	}
    }
 	# Success!
@@ -1257,7 +1287,7 @@ Again, this will automatically turn every incoming connection into a SSL socket.
 
 =head2 Events
 
-SimpleHTTP is so simple, there are only 7 events available.
+SimpleHTTP is so simple, there are only 8 events available.
 
 =over 4
 
@@ -1316,6 +1346,39 @@ SimpleHTTP is so simple, there are only 7 events available.
 
 =back
 
+=head2 Streaming with SimpleHTTP
+
+It's now possible to send data as a stream to clients (unbuffered and integrated in the 
+POE loop).
+
+Just create your sessions as usually and add a streaming event, this event will be triggered
+each time you set the $response to a streaming state:
+
+   # sets the response as streamed within our session with the stream event
+   $response->stream(
+      session  => 'HTTP_GET',
+      event    => 'GOT_STREAM'
+   );   
+
+This will call the GOT_STREAM event of the HTTP_GET session with as first arg (ARG0) bundled
+within a hash the wheel, request, response and id.
+
+You can now send data by chunks and either call yourself back (via POE) or shutdown when your 
+streaming is done (EOF for example).
+
+   sub GOT_STREAM {
+      my ( $kernel, $heap, $stream ) = @_[KERNEL, HEAP, ARG0];
+      
+      # $stream contains the wheel, the request, the response 
+      # and an id associated the the wheel
+      $stream->{'wheel'}->put("Hello World\n");
+   
+      # lets go on streaming ... with some delay actually but
+      # that should be a post unless the client needs the data
+      # slowly ..
+      POE::Kernel->delay('GOT_STREAM', 1, $stream );
+   }
+
 =head2 SimpleHTTP Notes
 
 This module is very picky about capitalization!
